package com.example.potatomemnap;

import java.io.Serializable;
import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.hardware.Camera;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.ShutterCallback;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.TextView;
import android.widget.Toast;

public class CameraActivity extends Activity {

	MediaPreview mediaPreview;
	// DrawView drawView;
	FrameLayout frameLayout;
	Button capImageBtn;
	Camera cam;
	MediaHandler mediaHandler;
	DBHandler dbhandler;
	GPSHandler gpshandler;
	LocationManager locationManager;

	private boolean lightOn = false;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		System.gc();
		// turn the title OFF
		requestWindowFeature(Window.FEATURE_NO_TITLE);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_camera, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.camera_exit:
			Intent intent = new Intent(Intent.ACTION_MAIN);
			intent.addCategory(Intent.CATEGORY_HOME);
			intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			System.gc();
			startActivity(intent);
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	public void load() {
		locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
		mediaHandler = new MediaHandler();
		dbhandler = new DBHandler(this.getApplicationContext());
		int rotate = mediaHandler.correctOrientation(
				this.getApplicationContext(), mediaHandler);

		// create new back face camera
		cam = Camera.open();

		setContentView(R.layout.activity_camera);

		// TextViews to show the lat/lng, to be removed
		TextView latView = (TextView) findViewById(R.id.latView);
		TextView lngView = (TextView) findViewById(R.id.lngView);
		gpshandler = new GPSHandler(this.getApplicationContext(),
				this.mediaHandler, this.locationManager, latView, lngView);

		frameLayout = (FrameLayout) findViewById(R.id.preview);
		// If the camera was received, create the app
		if (cam != null) {
			// Create a new camera view and add it to the layout
			mediaPreview = new MediaPreview(this, cam);
			frameLayout.addView(mediaPreview);

			// set the orientation
			cam.setDisplayOrientation(rotate);
		}

		// listener for flash button
		findViewById(R.id.flashBtn).setOnClickListener(
				new View.OnClickListener() {
					// actions to perform
					public void onClick(View v) {
						Parameters p = cam.getParameters();
						if (lightOn == false) {
							p.setFlashMode(Parameters.FLASH_MODE_TORCH);
							Log.d("cam act", "here going to on ");
							//cam.startPreview();
							lightOn = true;
						} else {
							p.setFlashMode(Parameters.FLASH_MODE_OFF);
							Log.d("cam act", "here going to off ");
							lightOn = false;
						}
						Log.d("cam act", "flash = " + p.getFlashMode());
						cam.setParameters(p);
					}
				});

		// listener for capture image button
		findViewById(R.id.capImageBtn).setOnClickListener(
				new View.OnClickListener() {
					// actions to perform
					public void onClick(View v) {
						mediaPreview.mCamera.takePicture(shutterCallback,
								rawCallback, jpegCallback);
					}
				});

		// listener for media(cam/vid) toggle
		findViewById(R.id.switchVidBtn).setOnClickListener(
				new View.OnClickListener() {
					// actions to perform
					public void onClick(View v) {
						Intent videoIntent = new Intent(
								getApplicationContext(), VideoActivity.class);
						startActivity(videoIntent);
						Log.d("cam act", "pressed switch");
					}
				});
	}

	ShutterCallback shutterCallback = new ShutterCallback() {
		public void onShutter() {
			Log.d("cam", "onShutter'd");
		}
	};

	/** Handles data for raw picture */
	PictureCallback rawCallback = new PictureCallback() {
		public void onPictureTaken(byte[] data, Camera camera) {
			Log.d("cam", "onPictureTaken - raw");
		}
	};

	/** Handles data for jpeg picture */
	PictureCallback jpegCallback = new PictureCallback() {
		public void onPictureTaken(byte[] data, Camera camera) {

			final String mediaFile = mediaHandler.setUpMediaFile(
					getApplicationContext(), gpshandler.getGPSValid(),
					mediaHandler, MediaHandler.IMAGE, data);
			// initially desc is "" until user edit in next screen
			mediaHandler.setDescription("");
			// initially fav location is not fav until user edit in next screen
			mediaHandler.setIsFav(MediaHandler.NOT_FAV);
			// initially favLocName is "" until user edit in next screen
			mediaHandler.setFavLocName("");
			// scan media
			mediaHandler
					.triggerMediaScanner(mediaFile, getApplicationContext());

			// String result = "";
			// String[] proj = { MediaStore.Images.Media.DISPLAY_NAME,
			// MediaStore.Images.Media.DESCRIPTION,
			// MediaStore.Images.Media._ID };
			// String selection = MediaStore.Images.Media.DISPLAY_NAME + " = "
			// + '"' + mediaHandler.getFilePath() + '"';

			// Log.d("in cam act", "filepath = " + mediaHandler.getFilePath());
			// CursorLoader loader = new CursorLoader(getApplicationContext(),
			// Uri.parse(mediaHandler.getFilePath()), proj, selection,
			// null, null);
			// File file = new File(mediaHandler.getFilePath());
			// if (file.exists()) {
			// Log.d("in cam act", "exist");
			// } else {
			// Log.d("in cam act", "dont exist");
			// }
			// Uri uri = Uri.parse(file);
			// Uri thumbUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
			// // Uri uri = Uri.parse(mediaHandler.getFilePath());
			// Cursor cursor = getContentResolver().query(thumbUri, proj,
			// selection, null, null);
			// if (cursor.moveToFirst()) {
			// int column_index = cursor
			// .getColumnIndexOrThrow(MediaStore.Images.Media.DISPLAY_NAME);
			// result = cursor.getString(column_index);
			// int column_index_BID = cursor
			// .getColumnIndexOrThrow(MediaStore.Images.Media._ID);
			// String resultBID = cursor.getString(column_index_BID);
			// Log.d("in cam act", "result mediastore = " + resultBID);
			// Log.d("in cam act", "result mediastore = " + result);
			//
			// String testName =
			// "/mnt/sdcard/PotatoMemNap/GPSInvalid/Morning/Picture_awdadawdw.jpg";
			// ContentResolver cr = getContentResolver();
			// ContentValues values = new ContentValues();
			// values.put(MediaStore.Images.ImageColumns.DISPLAY_NAME,
			// testName);
			// String slectionUpdate = MediaStore.Images.Media._ID
			// + " = " + '"' + resultBID + '"';
			// cr.update(thumbUri, values, slectionUpdate, null);
			//
			// // String selection2 = MediaStore.Images.Media.DISPLAY_NAME
			// // + " = " + '"' + testName + '"';
			// Cursor cursor2 = getContentResolver().query(thumbUri, proj,
			// slectionUpdate, null, null);
			// if (cursor2.moveToFirst()) {
			// int column_index2 = cursor2
			// .getColumnIndexOrThrow(MediaStore.Images.Media.DISPLAY_NAME);
			// String result2 = "";
			// int column_index_BID2 = cursor2
			// .getColumnIndexOrThrow(MediaStore.Images.Media._ID);
			// String resultBID2 = cursor2.getString(column_index_BID2);
			// result2 = cursor2.getString(column_index2);
			// Log.d("in cam act", "result new BID mediastore = "
			// + resultBID2);
			// Log.d("in cam act", "result new name mediastore = "
			// + result2);
			// }
			//
			// }

			Toast.makeText(getApplicationContext(), "Picture saved.",
					Toast.LENGTH_SHORT).show();

			// get a list of media that is added as fav loc
			List<MediaHandler> mediaListWithFav = dbhandler.getAllFavMedia();
			Log.d("Camera Activity media_list_with_fav_size", ""
					+ mediaListWithFav.size());

			if (gpshandler.getGPSValid() == 0 && mediaListWithFav.size() > 0) {
				// if GPS is valid, calculate the distance from source to
				// dest to bound a radius
				for (int i = 0; i < mediaListWithFav.size(); i++) {
					// for each media with fav loc, compare the lat/lng with the
					// current media's lat/lng
					float calcGPSDist = gpshandler.calculateGPSDistance(
							mediaListWithFav.get(i).getLat(), mediaListWithFav
									.get(i).getLng());

					if (calcGPSDist < 40) {
						Toast.makeText(
								getApplicationContext(),
								"At " + mediaListWithFav.get(i).getFavLocName(),
								Toast.LENGTH_SHORT).show();
						Log.d("distance", "At "
								+ mediaListWithFav.get(i).getFavLocName());
					} else {
						Toast.makeText(getApplicationContext(),
								"Not near any favourites", Toast.LENGTH_SHORT)
								.show();
						Log.d("Camera Activity distance",
								"Not near any favourites");
					}

					Toast.makeText(getApplicationContext(),
							"Distance is: " + calcGPSDist, Toast.LENGTH_SHORT)
							.show();
					Log.d("Camera Activity distance", "" + calcGPSDist
							+ " t/f: " + (calcGPSDist < 10));
				}
			}

			// process the GPS coord
			gpshandler.processCoord(mediaHandler.getLat(),
					mediaHandler.getLng());

			// save information to EXIF
			// mediaHandler.saveToEXIF(mediaFile, gpshandler.getLatDegree(),
			// gpshandler.getLngDegree(), gpshandler.getLatMin(),
			// gpshandler.getLngMin(), gpshandler.getLatSec(),
			// gpshandler.getLngSec(), gpshandler.getsLatR(),
			// gpshandler.getsLngR());

			// add the current media to db
			dbhandler.addMedia(new MediaHandler(mediaHandler.getFilePath(),
					mediaHandler.getLat(), mediaHandler.getLng(), mediaHandler
							.getDescription(), mediaHandler.getIsFav(),
					mediaHandler.getFavLocName()));

			Log.d("Camera Activity image count",
					"count = " + dbhandler.getMediaCount());

			// printing the just inserted to db media info
			// MediaHandler dbImage = dbhandler.getMediaInfo(mediaHandler
			// .getFilePath().getPath());
			//
			// Log.d("db info at camera",
			// "path = " + dbImage.getFilePath().getPath() + " lat = "
			// + dbImage.getLat() + " lng = " + dbImage.getLng()
			// + " desc = " + dbImage.getDescription()
			// + "is fav = " + dbImage.getIsFav());

			// after saving and the necessaries, go to the savedMediaActivity
			Intent savedMediaIntent = new Intent(getApplicationContext(),
					SavedMediaActivity.class);
			// passing the information of current intent to the next intent
			savedMediaIntent.putExtra("mediaHandler",
					(Serializable) mediaHandler);
			savedMediaIntent.putExtra("mediaFile", mediaFile);

			startActivity(savedMediaIntent);
		}
	};

	@Override
	protected void onPause() {
		super.onPause();
		Log.d("cam pausing", "pausing");
		if (gpshandler != null) {
			gpshandler.onPause();
		}

		if (cam != null) {
			cam.release();
			cam = null;
		}

		System.gc();
		// this.finish();
	}

	@Override
	protected void onResume() {
		super.onResume();
		System.gc();
		Log.d("cam resuming", "resuming");
		load();
	}

	protected void onDestroy() {
		super.onDestroy();
		Log.d("camera destroying", "destroying");
	}
}
