package com.example.potatomemnap;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

public class DBHandler extends SQLiteOpenHelper implements Serializable {

	// Database Version
	private static final int DATABASE_VERSION = 1;

	// Database Name
	private static final String DATABASE_NAME = "potatoMemNap_DB";

	// Media table name
	private static final String TABLE_MEDIAINFO = "info";

	// Media Table Columns names
	private static final String KEY_ID = "id";
	private static final String KEY_PATH = "path";
	private static final String KEY_LAT = "lat";
	private static final String KEY_LNG = "lng";
	private static final String KEY_DESCRIPTION = "description";
	private static final String KEY_FAV = "fav";
	private static final String KEY_FAVLOCNAME = "favLocName";

	// Constructor for database object
	public DBHandler(Context context) {
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}

	// called when a DBObject is created
	@Override
	public void onCreate(SQLiteDatabase db) {
		// create the table
		String CREATE_MEDIAINFO_TABLE = "CREATE TABLE " + TABLE_MEDIAINFO + "("
				+ KEY_ID + " INTEGER PRIMARY KEY, " + KEY_PATH + " TEXT, "
				+ KEY_LAT + " TEXT, " + KEY_LNG + " TEXT, " + KEY_DESCRIPTION
				+ " TEXT, " + KEY_FAV + " INTEGER, " + KEY_FAVLOCNAME
				+ " TEXT " + ")";

		Log.d("create table", CREATE_MEDIAINFO_TABLE);
		db.execSQL(CREATE_MEDIAINFO_TABLE);
	}

	// used when structure of DB is changed
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// Drop older table if existed
		db.execSQL("DROP TABLE IF EXISTS " + TABLE_MEDIAINFO);

		// Create tables again
		onCreate(db);
	}

	// add new media
	public void addMedia(MediaHandler mediaHandler) {
		// open DB for reading and writting
		SQLiteDatabase db = this.getWritableDatabase();

		Log.d("DBHandler inserting",
				"insertig path = " + mediaHandler.getFilePath() + " lat = "
						+ mediaHandler.getLat() + " lng = "
						+ mediaHandler.getLat() + " desc = "
						+ mediaHandler.getDescription() + " is fav = "
						+ mediaHandler.getIsFav() + "favLocName = "
						+ mediaHandler.getFavLocName());

		// informations to be inserted: path,lat,lng,desc
		ContentValues values = new ContentValues();
		values.put(KEY_PATH, mediaHandler.getFilePath());
		values.put(KEY_LAT, mediaHandler.getLat());
		values.put(KEY_LNG, mediaHandler.getLng());
		values.put(KEY_DESCRIPTION, mediaHandler.getDescription());
		values.put(KEY_FAV, mediaHandler.getIsFav());
		values.put(KEY_FAVLOCNAME, mediaHandler.getFavLocName());

		// insert
		db.insert(TABLE_MEDIAINFO, null, values);

		Log.d("in DBHandler done inserting", "done insertig");
		db.close(); // Closing database connection
	}

	// retrieve single media
	public MediaHandler getMediaInfo(String path) {
		SQLiteDatabase db = this.getReadableDatabase();

		// db.query: 1st para = table name, 2nd param = info to retrive, 3rd
		// param = "where" clause in sql which "?" is replaced by the 4th param,
		// followed by groupby, orderby, having and limit
		Cursor cursor = db.query(TABLE_MEDIAINFO, new String[] { KEY_PATH,
				KEY_LAT, KEY_LNG, KEY_DESCRIPTION, KEY_FAV, KEY_FAVLOCNAME },
				KEY_PATH + "=?", new String[] { String.valueOf(path) }, null,
				null, null, null);
		// if there are >=1 records, go to the 1st record
		if (cursor != null)
			cursor.moveToFirst();

		File mediaFilePath = new File(cursor.getString(0));
		Log.d("retiving", "retrive mediapath = " + mediaFilePath.getPath());
		Log.d("retiving", "retrive lat = " + cursor.getDouble(1));
		Log.d("retiving", "retrive lng = " + cursor.getDouble(2));
		Log.d("retiving", "retrive desc = " + cursor.getString(3));
		Log.d("retiving", "retrive fav = " + cursor.getString(4));
		Log.d("retiving", "retrive favLocName = " + cursor.getString(5));

		// retrive the info and return as an media object
		MediaHandler mediaHandler = new MediaHandler(
				mediaFilePath.getAbsolutePath(), cursor.getDouble(1),
				cursor.getDouble(2), cursor.getString(3), cursor.getInt(4),
				cursor.getString(5));

		cursor.close();
		db.close();
		return mediaHandler;
	}

	// retrieve all media info
	public List<MediaHandler> getAllMedia() {
		List<MediaHandler> mediaList = new ArrayList<MediaHandler>();
		// sql query to select a everything from table
		String selectQuery = "SELECT * FROM " + TABLE_MEDIAINFO;

		SQLiteDatabase db = this.getWritableDatabase();
		// rawquery takes in sql statement unlike query which has a structured
		// manner
		Cursor cursor = db.rawQuery(selectQuery, null);

		// loop through all rows and adding to list
		if (cursor.moveToFirst()) {
			do {
				MediaHandler mediaHandler = new MediaHandler();
				File mediaFilePath = new File(cursor.getString(1));
				mediaHandler.setFilePath(mediaFilePath.getAbsolutePath());
				mediaHandler.setLat(cursor.getDouble(2));
				mediaHandler.setLng(cursor.getDouble(3));
				mediaHandler.setDescription(cursor.getString(4));
				mediaHandler.setIsFav(cursor.getInt(5));
				mediaHandler.setFavLocName(cursor.getString(6));
				// add item to list
				mediaList.add(mediaHandler);
			} while (cursor.moveToNext());
		}
		cursor.close();
		db.close();
		return mediaList;
	}

	// retrieve all media info that is a fav location
	public List<MediaHandler> getAllFavMedia() {
		List<MediaHandler> mediaList = new ArrayList<MediaHandler>();
		// sql query to select a everything from table
		String selectQuery = "SELECT * FROM " + TABLE_MEDIAINFO + " WHERE "
				+ TABLE_MEDIAINFO + "." + KEY_FAV + " = " + MediaHandler.IS_FAV;

		SQLiteDatabase db = this.getWritableDatabase();
		// rawquery takes in sql statement unlike query which has a structured
		// manner
		Cursor cursor = db.rawQuery(selectQuery, null);

		// loop through all rows and adding to list
		if (cursor.moveToFirst()) {
			do {
				MediaHandler mediaHandler = new MediaHandler();
				File mediaFilePath = new File(cursor.getString(1));
				mediaHandler.setFilePath(mediaFilePath.getAbsolutePath());
				mediaHandler.setLat(cursor.getDouble(2));
				mediaHandler.setLng(cursor.getDouble(3));
				mediaHandler.setDescription(cursor.getString(4));
				mediaHandler.setIsFav(cursor.getInt(5));
				mediaHandler.setFavLocName(cursor.getString(6));
				// add item to list
				mediaList.add(mediaHandler);
			} while (cursor.moveToNext());
		}
		db.close();
		return mediaList;
	}

	// get number of items
	public int getMediaCount() {
		String countQuery = "SELECT  * FROM " + TABLE_MEDIAINFO;
		SQLiteDatabase db = this.getReadableDatabase();
		Cursor cursor = db.rawQuery(countQuery, null);
		int mediaCount = cursor.getCount();
		cursor.close();

		db.close();
		return mediaCount;
	}

	// update
	public int updateMediaDesc(MediaHandler mediaHandler) {
		SQLiteDatabase db = this.getWritableDatabase();

		// columns that is to be updated, here, only desc
		ContentValues values = new ContentValues();
		values.put(KEY_DESCRIPTION, mediaHandler.getDescription());

		// update row
		int numRowAffected = db.update(TABLE_MEDIAINFO, values, KEY_PATH
				+ " = ?",
				new String[] { String.valueOf(mediaHandler.getFilePath()) });
		db.close();

		return numRowAffected;
	}

	// update
	public int updateMediaPath(MediaHandler mediaHandler, String oldPath) {
		SQLiteDatabase db = this.getWritableDatabase();

		// columns that is to be updated, here, only desc
		ContentValues values = new ContentValues();
		values.put(KEY_PATH, mediaHandler.getFilePath());

		// update row
		int numRowAffected = db.update(TABLE_MEDIAINFO, values, KEY_PATH
				+ " = ?", new String[] { String.valueOf(oldPath) });
		db.close();

		return numRowAffected;
	}

	// update
	public int updateMediaFavLocName(MediaHandler mediaHandler) {
		SQLiteDatabase db = this.getWritableDatabase();

		// columns that is to be updated, here, only desc
		ContentValues values = new ContentValues();
		values.put(KEY_FAVLOCNAME, mediaHandler.getFavLocName());

		// update row
		int numRowAffected = db.update(TABLE_MEDIAINFO, values, KEY_PATH
				+ " = ?",
				new String[] { String.valueOf(mediaHandler.getFilePath()) });
		db.close();

		return numRowAffected;
	}

	// update
	public int updateMediaFav(MediaHandler mediaHandler) {
		SQLiteDatabase db = this.getWritableDatabase();

		// columns that is to be updated, here, only desc
		ContentValues values = new ContentValues();
		values.put(KEY_FAV, mediaHandler.getIsFav());

		// update row
		int numRowAffected = db.update(TABLE_MEDIAINFO, values, KEY_PATH
				+ " = ?",
				new String[] { String.valueOf(mediaHandler.getFilePath()) });

		db.close();

		return numRowAffected;
	}

	// delete an media
	public void deleteMedia(MediaHandler mediaHandler) {
		SQLiteDatabase db = this.getWritableDatabase();
		db.delete(TABLE_MEDIAINFO, KEY_PATH + " = ?",
				new String[] { String.valueOf(mediaHandler.getFilePath()) });
		db.close();
	}

}
