package com.example.potatomemnap;

import android.app.Activity;
import android.content.Context;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
import android.widget.TextView;
import android.widget.Toast;

public class GPSHandler extends Activity implements LocationListener {

	MediaHandler imageHandler;
	private LocationManager locationManager;
	Context context;
	boolean GPSEnabled = true;
	int GPSValid = GPS_INVALID;
	TextView latView;
	TextView lngView;

	static final int GPS_VALID = 0;
	static final int GPS_INVALID = 1;

	private String latDegree, latMin, latSec, sLatR, lngDegree, lngMin, lngSec,
			sLngR;

	private double testLat = 1.3104211678728461;
	private double testLng = 103.90087947249413;

	public GPSHandler(Context context, MediaHandler imageHandler,
			LocationManager locationManager, TextView latView, TextView lngView) {
		this.imageHandler = imageHandler;
		this.context = context;
		this.locationManager = locationManager;
		this.latView = latView;
		this.lngView = lngView;

		this.imageHandler.setLat(-1);
		this.imageHandler.setLng(-1);

		// this line defines how frequent to update, 2nd parameter is the
		// frequency in time to update, 3rd parameter is frequency in distance
		// to update, 4th parameter is the listener
		locationManager.requestLocationUpdates(locationManager.GPS_PROVIDER, 0,
				0, this);
		GPSEnabled = this.checkGPSEnable(context, locationManager);
	}

	public boolean checkGPSEnable(Context context,
			LocationManager locationManager) {
		boolean enabledGPS = locationManager
				.isProviderEnabled(LocationManager.GPS_PROVIDER);

		if (!enabledGPS) {
			Toast.makeText(context, "GPS is not enabled", Toast.LENGTH_SHORT)
					.show();
			return false;
		} else {
			return true;
		}
	}

	// convert coord to DMS format and get reference for EXIF
	protected void processCoord(double coordLat, double coordLng) {
		// get the decimal portion of coord
		double deciNum = coordLat % 1;
		// get the whole number part
		int wholeNum = (int) coordLat;

		// process latitude to dms
		// define latDegree
		latDegree = String.valueOf(wholeNum);

		// for minute, we want the decimal portion of coord * 60
		coordLat = deciNum * 60;
		// get the decimal portion
		deciNum = coordLat % 1;
		// get the whole number part
		wholeNum = (int) coordLat;

		// define latMin
		latMin = String.valueOf(wholeNum);

		// for seconds, we want the decimal portion of the minutes *60
		coordLat = deciNum * 60;
		// get the whole number portion of it
		wholeNum = (int) coordLat;

		// define latSec
		latSec = String.valueOf(wholeNum);
		// end lat deci to dms

		// process long to dms
		deciNum = coordLng % 1;
		wholeNum = (int) coordLng;

		lngDegree = String.valueOf(wholeNum);

		coordLng = deciNum * 60;
		deciNum = coordLng % 1;
		wholeNum = (int) coordLng;

		lngMin = String.valueOf(wholeNum);

		coordLng = deciNum * 60;
		wholeNum = (int) coordLng;

		lngSec = String.valueOf(wholeNum);
		// end long deci to dms

		if (coordLat < 0) {
			sLatR = "S";
		} else {
			sLatR = "N";
		}

		if (coordLng < 0) {
			sLngR = "W";
		} else {
			sLngR = "E";
		}
	}

	// this method obtains the distance between 2 locations from GPS
	protected float calculateGPSDistance(double calcLat, double calcLng) {
		// create the source location
		Location source = new Location("source");

		// source = lat/lng of info in DB fav loc
		source.setLatitude(calcLat);
		source.setLongitude(calcLng);

		// create the destination location which is the current lat/lng
		Location dest = new Location("dest");
		dest.setLatitude(this.imageHandler.getLat());
		dest.setLongitude(this.imageHandler.getLng());

		Log.d("dist values", "slat: " + source.getLatitude() + " slng: "
				+ source.getLongitude() + " dlat: " + dest.getLatitude()
				+ " dlng: " + dest.getLongitude());

		// obtain the distance from souce to destination in meters and return
		// the value
		return source.distanceTo(dest);
	}

	/*
	 * LocationListener implements 4 methods, 1)onLocationChanged,
	 * 2)onStatusChanged, 3)onProviderEnabled ,4)onProviderDisabled This method
	 * defines the operation to be performed when a location is changed
	 */
	public void onLocationChanged(Location location) {
		this.setGPSValid(GPS_VALID);
		this.imageHandler.setLat(location.getLatitude());
		this.imageHandler.setLng(location.getLongitude());
		String text = "My location is: Lat = " + this.imageHandler.getLat()
				+ " Lgn = " + this.imageHandler.getLng() + " GPSValid is: "
				+ this.getGPSValid();

		latView.setText("" + imageHandler.getLat());
		lngView.setText("" + imageHandler.getLng());
		Log.d("GPS DATA", text);
	}

	// This method is called when status of service provider is changed such as
	// temporary unavailable to available
	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO Auto-generated method stub
	}

	// This method is called when service provider is enabled
	public void onProviderEnabled(String provider) {
		/*
		 * Toast.makeText(this, "Enabled new provider " + provider,
		 * Toast.LENGTH_SHORT).show();
		 */
	}

	// This method is called when service provider is disabled
	public void onProviderDisabled(String provider) {
		/*
		 * Toast.makeText(this, "Disabled provider " + provider,
		 * Toast.LENGTH_SHORT).show();
		 */
		this.setGPSValid(GPS_INVALID);
	}

	public void onPause() {
		this.locationManager.removeUpdates(this);
		locationManager = null;
	}

	public int getGPSValid() {
		return GPSValid;
	}

	public void setGPSValid(int gpsValid) {
		GPSValid = gpsValid;
	}

	public String getLatDegree() {
		return latDegree;
	}

	public void setLatDegree(String latDegree) {
		this.latDegree = latDegree;
	}

	public String getLatMin() {
		return latMin;
	}

	public void setLatMin(String latMin) {
		this.latMin = latMin;
	}

	public String getLatSec() {
		return latSec;
	}

	public void setLatSec(String latSec) {
		this.latSec = latSec;
	}

	public String getsLatR() {
		return sLatR;
	}

	public void setsLatR(String sLatR) {
		this.sLatR = sLatR;
	}

	public String getLngDegree() {
		return lngDegree;
	}

	public void setLngDegree(String lngDegree) {
		this.lngDegree = lngDegree;
	}

	public String getLngMin() {
		return lngMin;
	}

	public void setLngMin(String lngMin) {
		this.lngMin = lngMin;
	}

	public String getLngSec() {
		return lngSec;
	}

	public void setLngSec(String lngSec) {
		this.lngSec = lngSec;
	}

	public String getsLngR() {
		return sLngR;
	}

	public void setsLngR(String sLngR) {
		this.sLngR = sLngR;
	}

	public double getTestLat() {
		return testLat;
	}

	public void setTestLat(double testLat) {
		this.testLat = testLat;
	}

	public double getTestLng() {
		return testLng;
	}

	public void setTestLng(double testLng) {
		this.testLng = testLng;
	}

}
