package com.example.potatomemnap;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.ExifInterface;
import android.media.MediaScannerConnection;
import android.media.MediaScannerConnection.MediaScannerConnectionClient;
import android.net.Uri;
import android.os.Environment;
import android.provider.MediaStore;
import android.text.format.Time;
import android.util.Log;
import android.view.Display;
import android.view.WindowManager;

@SuppressWarnings("serial")
public class MediaHandler implements MediaScannerConnectionClient, Serializable {

	String path;
	double lat;
	double lng;
	String description;
	int isFav;
	String favLocName;

	static final int IS_FAV = 0;
	static final int NOT_FAV = 1;

	static final int IMAGE = 0;
	static final int VIDEO = 1;

	final String invalidMorningPath = Environment.getExternalStorageDirectory()
			.getAbsolutePath() + "/PotatoMemNap/GPSInvalid/Morning/";
	final String invalidAfternoonPath = Environment
			.getExternalStorageDirectory().getAbsolutePath()
			+ "/PotatoMemNap/GPSInvalid/Afternoon/";
	final String invalidNightPath = Environment.getExternalStorageDirectory()
			.getAbsolutePath() + "/PotatoMemNap/GPSInvalid/Night/";
	final String invalidLateNightPath = Environment
			.getExternalStorageDirectory().getAbsolutePath()
			+ "/PotatoMemNap/GPSInvalid/Late_Night/";
	final String GPSValidPath = Environment.getExternalStorageDirectory()
			.getAbsolutePath() + "/PotatoMemNap/GPSValid/";

	public MediaHandler() {

	}

	public MediaHandler(String path, double lat, double lng,
			String description, int isFav, String favLocName) {
		this.path = path;
		this.lat = lat;
		this.lng = lng;
		this.description = description;
		this.isFav = isFav;
		this.favLocName = favLocName;
	}

	// this method is used to define the path of the photo taken
	protected String setUpMediaFile(Context context, int GPSValid,
			MediaHandler mediaHandler, int mediaType, byte[] data) {
		// establish mediascanner connection
		MediaScannerConnection msc = new MediaScannerConnection(context, this);
		msc.connect();

		// set up all folders
		File invalidMorningFile = new File(invalidMorningPath);
		if (!invalidMorningFile.exists()) {
			invalidMorningFile.mkdirs();
		}

		File invalidAfternoonFile = new File(invalidAfternoonPath);
		if (!invalidAfternoonFile.exists()) {
			invalidAfternoonFile.mkdirs();
		}

		File invalidNightFile = new File(invalidNightPath);
		if (!invalidNightFile.exists()) {
			invalidNightFile.mkdirs();
		}

		File invalidLateNightFile = new File(invalidLateNightPath);
		if (!invalidLateNightFile.exists()) {
			invalidLateNightFile.mkdirs();
		}

		File GPSValidFile = new File(GPSValidPath);
		if (!GPSValidFile.exists()) {
			GPSValidFile.mkdirs();
		}
		// end of set up

		String fileName = "";

		int rotate = mediaHandler.correctOrientation(context, mediaHandler);

		// construct the name of the picture taken
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyymmddhhmmss");
		String date = dateFormat.format(new Date());

		// String mediaFilePath = "Picture_/Video_" + date + ".jpg/mp4";
		String mediaName = "";
		if (mediaType == MediaHandler.IMAGE) {
			mediaName = "Picture_" + date + ".jpg";
		} else {
			mediaName = "Video_" + date + ".mp4";
		}

		// determine the directory of the image depending on whether GPS signal
		// available and time of the day
		if (GPSValid == 0) {
			fileName = Environment.getExternalStorageDirectory().getPath()
					+ "/" + "PotatoMemNap" + "/" + "GPSValid" + "/" + mediaName;
		} else if (GPSValid == 1) {
			// Create Time object
			Time now = new Time();
			// get the current time
			now.setToNow();
			// get the hour of the current time
			int hourNow = now.hour;

			if (hourNow >= 6 && hourNow <= 11) {
				// morning
				fileName = Environment.getExternalStorageDirectory().getPath()
						+ "/" + "PotatoMemNap" + "/" + "GPSInvalid" + "/"
						+ "Morning" + "/" + mediaName;
			} else if (hourNow >= 12 && hourNow <= 17) {
				// afternoon
				fileName = Environment.getExternalStorageDirectory().getPath()
						+ "/" + "PotatoMemNap" + "/" + "GPSInvalid" + "/"
						+ "Afternoon" + "/" + mediaName;
			} else if (hourNow >= 18 && hourNow <= 23) {
				// night
				fileName = Environment.getExternalStorageDirectory().getPath()
						+ "/" + "PotatoMemNap" + "/" + "GPSInvalid" + "/"
						+ "Night" + "/" + mediaName;
			} else if (hourNow >= 0 && hourNow <= 5) {

				fileName = Environment.getExternalStorageDirectory().getPath()
						+ "/" + "PotatoMemNap" + "/" + "GPSInvalid" + "/"
						+ "Late_night" + "/" + mediaName;
			}
		}

		long timeTaken = System.currentTimeMillis();

		if (mediaType == MediaHandler.IMAGE) {
			ContentValues values = new ContentValues(9);
			values.put(MediaStore.Images.ImageColumns.DATE_TAKEN, timeTaken);
			values.put(MediaStore.Images.ImageColumns.TITLE, "PotatoMemNap");
			values.put(MediaStore.Images.ImageColumns.DISPLAY_NAME, fileName);
			// use the description field to keep info of isFav,favLocName,Desc,
			// separated by "|" i.e. NOT_FAV|HOME|SOME_DESC
			values.put(MediaStore.Images.ImageColumns.DESCRIPTION, "||");
			values.put(MediaStore.Images.ImageColumns.MIME_TYPE, "image/jpeg");
			values.put(MediaStore.Images.ImageColumns.ORIENTATION, rotate);
			values.put(MediaStore.Images.ImageColumns.LATITUDE,
					mediaHandler.getLat());
			values.put(MediaStore.Images.ImageColumns.LONGITUDE,
					mediaHandler.getLng());
			values.put(MediaStore.Images.ImageColumns.DATA, fileName);

			Uri uri = context
					.getContentResolver()
					.insert(android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
							values);

			OutputStream imageFileOS;

			try {
				Bitmap bitmap = BitmapFactory.decodeByteArray(data, 0,
						data.length);
				imageFileOS = context.getContentResolver()
						.openOutputStream(uri);
				bitmap.compress(Bitmap.CompressFormat.JPEG, 70, imageFileOS);
				imageFileOS.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			ContentValues values = new ContentValues(8);
			values.put(MediaStore.Video.VideoColumns.DATE_TAKEN, timeTaken);
			values.put(MediaStore.Images.ImageColumns.TITLE, "PotatoMemNap");
			values.put(MediaStore.Images.ImageColumns.DISPLAY_NAME, fileName);
			values.put(MediaStore.Video.VideoColumns.DESCRIPTION,
					"PotatoMemNap_Video");
			values.put(MediaStore.Video.VideoColumns.MIME_TYPE, "video/mp4");
			values.put(MediaStore.Video.VideoColumns.LATITUDE,
					mediaHandler.getLat());
			values.put(MediaStore.Video.VideoColumns.LONGITUDE,
					mediaHandler.getLng());
			values.put(MediaStore.Video.VideoColumns.DATA, fileName);
			values.put(MediaStore.Video.VideoColumns.WIDTH, 1280);
			values.put(MediaStore.Video.VideoColumns.HEIGHT, 720);
		}

		Log.d("in MediaHandler", "GPSvalid: " + GPSValid + "  path: "
				+ fileName);
		mediaHandler.setFilePath(fileName);

		return fileName;
	}

	// this method is used to trigger media scanner, allowing photos taken to be
	// read
	public void triggerMediaScanner(String mediaFile, Context context) {
		Intent mediaScanIntent = new Intent(
				Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
		// File f = new File(mediaFile);
		Uri contentUri = Uri.parse(mediaFile);
		mediaScanIntent.setData(contentUri);
		context.sendBroadcast(mediaScanIntent);
	}

	public String[] getDescriptionFromMediaStore(Context context,
			String filePath) {

		String resultDesc = "";
		String resultID = "";

		String[] proj = { MediaStore.Images.Media.DISPLAY_NAME,
				MediaStore.Images.Media.DESCRIPTION,
				MediaStore.Images.Media._ID };
		String selection = MediaStore.Images.Media.DISPLAY_NAME + " = " + '"'
				+ filePath + '"';
		Uri contentUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
		Cursor cursor = context.getContentResolver().query(contentUri, proj,
				selection, null, null);

		if (cursor.moveToFirst()) {
			int columnIndexDesc = cursor
					.getColumnIndexOrThrow(MediaStore.Images.Media.DESCRIPTION);
			resultDesc = cursor.getString(columnIndexDesc);

			int columnIndexID = cursor
					.getColumnIndexOrThrow(MediaStore.Images.Media._ID);
			resultID = cursor.getString(columnIndexID);

			Log.d("in mediaHandler", "result mediastoreID = " + resultID);
			Log.d("in mediaHandler", "result mediastoreDESC = " + resultDesc);

			String[] fResult = { resultID, resultDesc };
			return fResult;
		} else {
			String[] fResult = null;
			return fResult;
		}
		// String testName =
		// "/mnt/sdcard/PotatoMemNap/GPSInvalid/Morning/Picture_awdadawdw.jpg";
		// ContentResolver cr = getContentResolver();
		// ContentValues values = new ContentValues();
		// values.put(MediaStore.Images.ImageColumns.DISPLAY_NAME,
		// testName);
		// String slectionUpdate = MediaStore.Images.Media._ID
		// + " = " + '"' + resultBID + '"';
		// cr.update(thumbUri, values, slectionUpdate, null);
		//
		// // String selection2 = MediaStore.Images.Media.DISPLAY_NAME
		// // + " = " + '"' + testName + '"';
		// Cursor cursor2 = getContentResolver().query(thumbUri, proj,
		// slectionUpdate, null, null);
		// if (cursor2.moveToFirst()) {
		// int column_index2 = cursor2
		// .getColumnIndexOrThrow(MediaStore.Images.Media.DISPLAY_NAME);
		// String result2 = "";
		// int column_index_BID2 = cursor2
		// .getColumnIndexOrThrow(MediaStore.Images.Media._ID);
		// String resultBID2 = cursor2.getString(column_index_BID2);
		// result2 = cursor2.getString(column_index2);
		// Log.d("in cam act", "result new BID mediastore = "
		// + resultBID2);
		// Log.d("in cam act", "result new name mediastore = "
		// + result2);
		// }
	}

	// ///////filePath can be removed later/////////
	public void updateDescFromMediaStore(Context context, String rowID,
			String newValue, String filePath) {
		ContentResolver cr = context.getContentResolver();
		ContentValues values = new ContentValues();
		Uri contentUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
		values.put(MediaStore.Images.ImageColumns.DESCRIPTION, newValue);
		String slectionUpdate = MediaStore.Images.Media._ID + " = " + '"'
				+ rowID + '"';
		cr.update(contentUri, values, slectionUpdate, null);

		// check result, to be removed
		this.getDescriptionFromMediaStore(context, filePath);
	}

	// save info to EXIF
	protected void saveToEXIF(String path, String latDegree, String lngDegree,
			String latMin, String lngMin, String latSec, String lngSec,
			String sLatR, String sLngR) {
		try {
			// instantiate exifinterface
			ExifInterface exifInterface = new ExifInterface(path);
			String fLat = "";
			String fLng = "";
			String fLatR = "";
			String fLngR = "";

			// values for the various EXIF interface
			fLat = latDegree + "/1," + latMin + "/1," + latSec + "/1";
			fLng = lngDegree + "/1," + lngMin + "/1," + lngSec + "/1";
			fLatR = sLatR;
			fLngR = sLngR;

			// set the attributes for lat long
			exifInterface.setAttribute(ExifInterface.TAG_GPS_LATITUDE, fLat);

			exifInterface.setAttribute(ExifInterface.TAG_GPS_LONGITUDE, fLng);

			exifInterface.setAttribute(ExifInterface.TAG_GPS_LATITUDE_REF,
					fLatR);

			exifInterface.setAttribute(ExifInterface.TAG_GPS_LONGITUDE_REF,
					fLngR);

			exifInterface.setAttribute(ExifInterface.TAG_ORIENTATION,
					String.valueOf(90));

			// save the attributes
			exifInterface.saveAttributes();
			// this shows how to retrive info from exif
			Log.d("in mediaHandler EXIF DATA",
					"lat: "
							+ exifInterface
									.getAttribute(ExifInterface.TAG_GPS_LONGITUDE)
							+ " lng: "
							+ exifInterface
									.getAttribute(ExifInterface.TAG_GPS_LONGITUDE)
							+ " orientation: "
							+ exifInterface
									.getAttribute(ExifInterface.TAG_ORIENTATION));
		} catch (IOException e) {
			Log.d("Error", e.getMessage());
			return;
		}
	}

	public int getMediaType(String path) {
		String mediaType = path.substring(path.indexOf(".") + 1);
		if (mediaType.equalsIgnoreCase("jpg")) {
			return MediaHandler.IMAGE;
		} else {
			return MediaHandler.VIDEO;
		}
	}

	// rotation 0 = potrait(0), tilt right to landscape = 270(3), tile left
	// to landscape = 90(1), tilt left left to potrait = 180(2)
	public int correctOrientation(Context context, MediaHandler mediaHandler) {
		// ExifInterface exif = null;
		// try {
		// exif = new ExifInterface(mediaHandler.getFilePath());
		// orientation = Integer.parseInt(exif
		// .getAttribute(ExifInterface.TAG_ORIENTATION));
		//
		// Log.d("mediaHandler",
		// "path = " + mediaHandler.getFilePath() + " orientation = "
		// + exif.getAttribute(ExifInterface.TAG_ORIENTATION));
		// } catch (IOException e) {
		// e.printStackTrace();
		// }

		int orientation = 0;

		Display display = ((WindowManager) context
				.getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay();

		orientation = display.getRotation();
		int rotate = 0;
		switch (orientation) {
		case 0:
			rotate = 90;
			break;
		case 1:
			rotate = 0;
			break;
		case 2:
			rotate = 270;
			break;
		case 3:
			rotate = 180;
			break;
		}
		Log.d("ORIENTATION_TEST", "getOrientation(): " + orientation
				+ " return rotate = " + rotate);
		return rotate;
	}

	public Bitmap decodeSampledBitmapFromUri(String path, int reqWidth,
			int reqHeight) {
		Bitmap bm = null;

		// First decode with inJustDecodeBounds=true to check dimensions
		final BitmapFactory.Options options = new BitmapFactory.Options();
		options.inJustDecodeBounds = true;
		BitmapFactory.decodeFile(path, options);

		// Calculate inSampleSize
		options.inSampleSize = calculateInSampleSize(options, reqWidth,
				reqHeight);

		// Decode bitmap with inSampleSize set
		options.inJustDecodeBounds = false;
		bm = BitmapFactory.decodeFile(path, options);

		return bm;
	}

	// this method helps to decode the media in a memory saving manner
	public int calculateInSampleSize(BitmapFactory.Options options,
			int reqWidth, int reqHeight) {
		// Raw height and width of image
		final int height = options.outHeight;
		final int width = options.outWidth;
		int inSampleSize = 1;

		if (height > reqHeight || width > reqWidth) {
			if (width > height) {
				inSampleSize = Math.round((float) height / (float) reqHeight);
			} else {
				inSampleSize = Math.round((float) width / (float) reqWidth);
			}
		}
		return inSampleSize;
	}

	public void onMediaScannerConnected() {
		Log.d("scanner connected", "scanner connected");
	}

	public void onScanCompleted(String path, Uri uri) {
		Log.d("scanned comeplete", " scan complete");
	}

	public double getLat() {
		return lat;
	}

	public void setLat(double lat) {
		this.lat = lat;
	}

	public double getLng() {
		return lng;
	}

	public void setLng(double lng) {
		this.lng = lng;
	}

	public String getFilePath() {
		return path;
	}

	public void setFilePath(String path) {
		this.path = path;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public int getIsFav() {
		return isFav;
	}

	public void setIsFav(int isFav) {
		this.isFav = isFav;
	}

	public String getFavLocName() {
		return favLocName;
	}

	public void setFavLocName(String favLocName) {
		this.favLocName = favLocName;
	}

}
