package com.example.potatomemnap;

import java.io.IOException;

import android.content.Context;
import android.graphics.ImageFormat;
import android.hardware.Camera;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class MediaPreview extends SurfaceView implements SurfaceHolder.Callback {

	private SurfaceHolder mHolder;
	public Camera mCamera;

	public MediaPreview(Context context, Camera camera) {
		super(context);

		mCamera = camera;
		// get the holder
		mHolder = getHolder();
		// implement callback to get notified of surfacechanges
		mHolder.addCallback(this);
	}

	// this method is called when changes are made to the surface
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {

		// if surface does not exist, do nothing
		if (mHolder.getSurface() == null) {
			return;
		}

		// in cases where this happen, stop preview
		try {
			mCamera.stopPreview();
		} catch (Exception e) {
			e.printStackTrace();
		}

		// after making corresponding changes, start preview with new settings
		try {
			mCamera.setPreviewDisplay(mHolder);
			Camera.Parameters parameters = mCamera.getParameters();
			Camera.Size size = getBestPreviewSize(width, height, parameters);
			Camera.Size pictureSize = getSmallestPictureSize(parameters);

			if (size != null && pictureSize != null) {
				parameters.setPreviewSize(1920, 1080);
				parameters.setPictureSize(1600, 1200);
				parameters.setPictureFormat(ImageFormat.JPEG);
				mCamera.setParameters(parameters);
			}
			mCamera.startPreview();
		} catch (Exception e) {
			Log.d("CameraView",
					"Error starting camera preview: " + e.getMessage());
		}
	}

	// this method is called when surface is created
	public void surfaceCreated(SurfaceHolder holder) {
		try {
			// set the surface to play the preview
			mCamera.setPreviewDisplay(holder);
			mCamera.startPreview();
		} catch (IOException e) {
			Log.d("CameraView",
					"Error setting camera preview: " + e.getMessage());
		}
	}

	public void surfaceDestroyed(SurfaceHolder holder) {
	}

	private Camera.Size getBestPreviewSize(int width, int height,
			Camera.Parameters parameters) {
		Camera.Size result = null;

		for (Camera.Size size : parameters.getSupportedPreviewSizes()) {
			if (size.width <= width && size.height <= height) {
				if (result == null) {
					result = size;
				} else {
					int resultArea = result.width * result.height;
					int newArea = size.width * size.height;

					if (newArea > resultArea) {
						result = size;
					}
				}
			}
		}

		return (result);
	}

	private Camera.Size getSmallestPictureSize(Camera.Parameters parameters) {
		Camera.Size result = null;

		for (Camera.Size size : parameters.getSupportedPictureSizes()) {
			if (result == null) {
				result = size;
			} else {
				int resultArea = result.width * result.height;
				int newArea = size.width * size.height;

				if (newArea < resultArea) {
					result = size;
				}
			}
		}

		return (result);
	}

	public SurfaceHolder getmHolder() {
		return mHolder;
	}

	public void setmHolder(SurfaceHolder mHolder) {
		this.mHolder = mHolder;
	}

}
