package com.example.potatomemnap;

import java.io.IOException;
import java.io.Serializable;
import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.hardware.Camera;
import android.hardware.Camera.Parameters;
import android.location.LocationManager;
import android.media.CamcorderProfile;
import android.media.MediaRecorder;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.TextView;
import android.widget.Toast;

public class VideoActivity extends Activity {

	Button recVidBtn;
	MediaRecorder mediaRecorder;
	MediaPreview mediaPreview;
	// used to track whether is it recording now
	boolean recording;
	MediaHandler mediaHandler;
	DBHandler dbhandler;
	GPSHandler gpshandler;
	LocationManager locationManager;
	Camera cam;
	FrameLayout frameLayout;
	String vidFile;

	private boolean lightOn = false;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		System.gc();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_video, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.video_exit:
			Intent intent = new Intent(Intent.ACTION_MAIN);
			intent.addCategory(Intent.CATEGORY_HOME);
			intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			System.gc();
			startActivity(intent);
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	public void load() {
		locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
		mediaHandler = new MediaHandler();
		dbhandler = new DBHandler(this.getApplicationContext());

		recording = false;

		mediaRecorder = new MediaRecorder();
		cam = Camera.open();
		cam.setDisplayOrientation(90);

		setContentView(R.layout.activity_video);

		TextView latView = (TextView) findViewById(R.id.latView);
		TextView lngView = (TextView) findViewById(R.id.lngView);
		gpshandler = new GPSHandler(this.getApplicationContext(),
				this.mediaHandler, this.locationManager, latView, lngView);

		frameLayout = (FrameLayout) findViewById(R.id.videoview);

		// If the camera was received, create the app
		if (cam != null) {
			// Create a new camera view and add it to the layout
			mediaPreview = new MediaPreview(this, cam);
			frameLayout.addView(mediaPreview);

			// Create a new draw view and add it to the layout
			// drawView = new DrawView(this);
			// frameLayout.addView(drawView);
		}

		// listener for flash button
		findViewById(R.id.flashBtn).setOnClickListener(
				new View.OnClickListener() {
					// actions to perform
					public void onClick(View v) {
						Parameters p = cam.getParameters();
						if (lightOn == false) {
							p.setFlashMode(Parameters.FLASH_MODE_TORCH);
							Log.d("cam act", "here going to on ");
							// cam.startPreview();
							lightOn = true;
						} else {
							p.setFlashMode(Parameters.FLASH_MODE_OFF);
							Log.d("cam act", "here going to off ");
							lightOn = false;
						}
						Log.d("cam act", "flash = " + p.getFlashMode());
						cam.setParameters(p);
					}
				});

		recVidBtn = (Button) findViewById(R.id.recVidBtn);
		// listener for record video button
		recVidBtn.setOnClickListener(new View.OnClickListener() {
			// actions to perform
			public void onClick(View v) {
				// recording now, next click will be stop recording
				if (recording) {
					// add the current media to db
					dbhandler.addMedia(new MediaHandler(mediaHandler
							.getFilePath(), mediaHandler.getLat(), mediaHandler
							.getLng(), mediaHandler.getDescription(),
							mediaHandler.getIsFav(), mediaHandler
									.getFavLocName()));

					// get a list of media that is added as fav loc
					List<MediaHandler> mediaListWithFav = dbhandler
							.getAllFavMedia();

					if (gpshandler.getGPSValid() == 0
							&& mediaListWithFav.size() > 0) {
						// if GPS is valid, calculate the distance from source
						// to dest to bound a radius
						for (int i = 0; i < mediaListWithFav.size(); i++) {
							// for each media with fav loc, compare the lat/lng
							// with the current media's lat/lng
							float calcGPSDist = gpshandler
									.calculateGPSDistance(
											mediaListWithFav.get(i).getLat(),
											mediaListWithFav.get(i).getLng());

							if (calcGPSDist < 20) {
								Toast.makeText(getApplicationContext(),
										"at home", Toast.LENGTH_SHORT).show();
								Log.d("distance", "at home");
							} else {
								Toast.makeText(getApplicationContext(),
										"NOT at home", Toast.LENGTH_SHORT)
										.show();
								Log.d("distance", "NOT at home");
							}

							Toast.makeText(getApplicationContext(),
									"Distance is: " + calcGPSDist,
									Toast.LENGTH_SHORT).show();
							Log.d("In video activity", "" + calcGPSDist
									+ "t/f: " + (calcGPSDist < 10));
						}
					}

					// after stop recording, go to the savedMediaIntent
					Intent savedMediaIntent = new Intent(
							getApplicationContext(), SavedMediaActivity.class);
					// passing the information of current intent to the next
					// intent
					savedMediaIntent.putExtra("mediaHandler",
							(Serializable) mediaHandler);
					savedMediaIntent.putExtra("mediaFile",
							(Serializable) vidFile);
					dbhandler.close();

					startActivity(savedMediaIntent);
				} else {
					// not recording now, next click will be start recording
					initMediaRecorder();
					mediaRecorder.start();
					recording = true;
					recVidBtn.setText("Stop Record");
				}
			}
		});

		// listener for switch to cam button
		findViewById(R.id.switchCamBtn).setOnClickListener(
				new View.OnClickListener() {
					// actions to perform
					public void onClick(View v) {
						dbhandler.close();
						Intent camIntent = new Intent(getApplicationContext(),
								CameraActivity.class);
						startActivity(camIntent);
					}
				});
	}

	// this method is called to initialise the requried preparations
	private void initMediaRecorder() {
		// set up the filepath and saving the video
		vidFile = mediaHandler.setUpMediaFile(getApplicationContext(),
				gpshandler.getGPSValid(), mediaHandler, MediaHandler.VIDEO,
				null);
		// initially desc is "" until user edit in next screen
		mediaHandler.setDescription("");
		// initially fav location is not fav until user edit in next screen
		mediaHandler.setIsFav(MediaHandler.NOT_FAV);
		// initially favLocName is "" until user edit in next screen
		mediaHandler.setFavLocName("");

		try {
			// stop the preview
			if (cam != null) {
				cam.stopPreview();
			}
			// release the camera for mediaRecorder to use
			cam.unlock();
			Parameters para = cam.getParameters();
			Log.d("Video Activity", para.toString());

			int rotate = mediaHandler.correctOrientation(
					this.getApplicationContext(), mediaHandler);

			// note that changing the order of the following might cause the
			// video recording to not work
			mediaRecorder.setCamera(cam);
			mediaRecorder.setAudioSource(MediaRecorder.AudioSource.DEFAULT);
			mediaRecorder.setVideoSource(MediaRecorder.VideoSource.DEFAULT);
			CamcorderProfile camcorderProfileQuality = CamcorderProfile
					.get(CamcorderProfile.QUALITY_1080P);
			mediaRecorder.setProfile(camcorderProfileQuality);
			mediaRecorder.setOrientationHint(rotate);
			mediaRecorder.setOutputFile(vidFile);
			mediaRecorder.setMaxDuration(60000); // Set max duration 60 sec.
			mediaRecorder.setPreviewDisplay(mediaPreview.getmHolder()
					.getSurface());
			mediaRecorder.setVideoSize(1280, 720);
			mediaRecorder.prepare();
		} catch (IllegalStateException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected void onPause() {
		super.onPause();
		if (gpshandler != null) {
			gpshandler.onPause();
		}

		if (mediaRecorder != null) {
			mediaRecorder.reset();
			mediaRecorder.release();
		}

		if (cam != null) {
			cam.release();
			cam = null;
		}

		if (dbhandler != null) {
			dbhandler.close();
			dbhandler = null;
		}
		Log.d("vid pausing", "pausing");
		// finish();
	}

	@Override
	protected void onResume() {
		super.onResume();
		Log.d("vid resuming", "resuming");
		load();
	}

	protected void onDestroy() {
		super.onDestroy();
		Log.d("vid destroying", "destroying");
	}
}